"""
A TCP client that can send and receive data from servers asynchronously
"""

import socket
import thread
from time import sleep
from defaultRecvHandler import defaultRecvHandler

class TCPClient:
  def __init__(self, ip, port):
    self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    self.sendFlag = 0
    self.stopSendThread = 0
    self.stopRecvThread = 0
    self.sendBuffer = []
    self.sendLock = thread.allocate_lock()
    self.granularity = 1.0 / 60.0
    self.serverAddr = (ip, port)
    self.recvBuffSize = 2048
    self.recvHandler = defaultRecvHandler
    self.recvFunc = self.__recv

  # Eventually connect for network interfaces that need it and start sending and receiving threads
  def start(self):
    # Connect to server
    self.socket.connect(self.serverAddr)
    # Start sending and receiving threads
    self.stopSendThread = 0    
    self.stopRecvThread = 0    
    thread.start_new_thread(self.__send, ())
    thread.start_new_thread(self.recvFunc, ())
    
  # Stop the sending and receiving threads
  def stop(self):
    self.stopSendThread = 1
    self.stopRecvThread = 1
    
  # Place some data in the sending queue
  def send(self, data):
    self.sendLock.acquire()
    self.sendBuffer.append(data)
    self.sendFlag = 1
    self.sendLock.release()

  ### Private methods ###
  
  # The sending loop
  def __send(self):
    while not self.stopSendThread:
      if self.sendFlag:
        self.sendLock.acquire()
        sendBuffer = self.sendBuffer
        self.sendBuffer = []
        self.sendFlag = 0
        self.sendLock.release()
        for data in sendBuffer:
          self.socket.send(data)
      sleep(self.granularity)

  # The receiving loop
  def __recv(self):
    while not self.stopRecvThread:
      data = self.socket.recv(self.recvBuffSize)
      if data:
        a = self.recvHandler()
        a.handle(data)
      sleep(self.granularity)
